@@ -2,7 +2,7 @@
  * File:				utils.h
  * Created:				August 2007
  * Created by:			Axel von Bertoldi
- * Last Modified:		January 2008
+ * Last Modified:		August 2008
  * Last Modified by:	Axel von Bertoldi
  * (C) 2005-2008		Axel von Bertoldi
  *
@@ -31,10 +31,8 @@
 
 #include "config.h"
 
-#define DEBUG 1
-
 gboolean	utils_check_gerror (GError **error);
-void		utils_show_dialog (gchar *title, gchar *message, GtkMessageType type);
+void		utils_show_dialog (const gchar *title, const gchar *message, GtkMessageType type);
 GtkWidget*	utils_get_scaled_image_from_file (gchar *file_name, int size);
 GSList*		g_slist_swap_data (GSList *list, guint index);
 gint		utils_sort_alpha (const gchar **s1, const gchar **s2);
